@@ -29,36 +29,63 @@ def rate_limit(f, tries=30):
                 errors = 0
                 return resp
             elif resp.status_code == 429:
-                reset = int(resp.headers["x-rate-limit-reset"])
-                now = time.time()
 
-                # The time to sleep depends on having an accurate system time,
-                # so check to see if there's something really bad happening
-                # to warn the user.
-                target_sleep_seconds = reset - now
+                # Check the headers, and try to infer why we're hitting the
+                # rate limit. Because the search/all endpoints also have a
+                # 1r/s rate limit that isn't obvious in the headers, we need
+                # to infer the reason for the rate limit. Note that this is
+                # included to help debug problems with multiple concurrent
+                # clients - this shouldn't be hit in normal of operation of a
+                # single twarc client.
+                remaining = int(resp.headers["x-rate-limit-remaining"])
 
-                # Never sleep longer than 15 minutes, as that is the basis for
-                # all of the read time based rate limits in the Twitter API
-                seconds = min(901, max(10, (target_sleep_seconds + 10)))
-
-                if target_sleep_seconds >= 900:
-                    # If we need to sleep for more than a rate limit period, the
-                    # system clock could be wrong.
+                # If we have a 429 rate limit, but there are remaining calls for
+                # this endpoint, we've probably hit the 1r/s limit.
+                if remaining:
                     log.warning(
-                        "Detected overlong sleep interval - is your system clock accurate? "
-                        "An accurate system time is needed to calculate how long to sleep for, "
-                        "and data collection might be slowed."
-                    )
-                elif target_sleep_seconds < 0:
-                    # If we need to sleep for negative time something weird might be up.
-                    log.warning(
-                        "Detected negative sleep interval - is your system clock accurate? "
-                        "If your system time is running fast, rate limiting may not be "
-                        "effective."
+                        "Hit the 1 request/second rate limit, sleeping for 10 seconds. "
+                        "This shouldn't happen with normal usage of twarc, and may indicate "
+                        "multiple clients interacting with the Twitter API at the "
+                        "same time."
                     )
+                    time.sleep(10)
+                    continue
+
+                # Just a regular 15 minute window rate limit.
+                else:
+                    reset = int(resp.headers["x-rate-limit-reset"])
+                    now = time.time()
+
+                    # The time to sleep depends on having an accurate system time,
+                    # so check to see if there's something really bad happening
+                    # to warn the user.
+                    target_sleep_seconds = reset - now
+
+                    # Never sleep longer than 15 minutes, as that is the basis for
+                    # all of the read time based rate limits in the Twitter API
+                    seconds = min(901, max(10, (target_sleep_seconds + 10)))
+
+                    if target_sleep_seconds >= 900:
+                        # If we need to sleep for more than a rate limit period, the
+                        # system clock could be wrong.
+                        log.warning(
+                            "Detected overlong sleep interval - is your system clock accurate? "
+                            "An accurate system time is needed to calculate how long to sleep for, "
+                            "and data collection might be slowed. "
+                            f"The rate limit resets at {reset} and the current time is {now}."
+                        )
+                    elif target_sleep_seconds < 0:
+                        # If we need to sleep for negative time something weird might be up.
+                        log.warning(
+                            "Detected negative sleep interval - is your system clock accurate? "
+                            "If your system time is running fast, rate limiting may not be "
+                            "effective. "
+                            f"The rate limit resets at {reset} and the current time is {now}."
+                        )
+
+                    log.warning("rate limit exceeded: sleeping %s secs", seconds)
+                    time.sleep(seconds)
 
-                log.warning("rate limit exceeded: sleeping %s secs", seconds)
-                time.sleep(seconds)
             elif resp.status_code >= 500:
                 errors += 1
                 if errors > tries:
@@ -175,7 +202,6 @@ class cli_api_error:
 def requires_app_auth(f):
     """
     Ensure that application authentication is set for calls that only work in that mode.
-
     """
 
     @wraps(f)
@@ -197,6 +223,75 @@ class InvalidAuthType(Exception):
     """
 
 
+class FileLineProgressBar(tqdm):
+    """
+    A progress bar based on input file line count. Counts an input file by lines.
+    This tries to read the entire file and count newlines in a robust way.
+    """
+
+    def __init__(self, infile, outfile, **kwargs):
+        disable = False if "disable" not in kwargs else kwargs["disable"]
+        if infile is not None and (infile.name == "<stdin>"):
+            disable = True
+        if outfile is not None and (outfile.name == "<stdout>"):
+            disable = True
+        kwargs["disable"] = disable
+        kwargs["miniters"] = 1
+        kwargs[
+            "bar_format"
+        ] = "{l_bar}{bar}| Processed {n_fmt}/{total_fmt} lines of input file [{elapsed}<{remaining}, {rate_fmt}{postfix}]"
+
+        # Warn for large (> 1 GB) input files:
+        if (os.stat(infile.name).st_size / (1024 * 1024 * 1024)) > 1:
+            click.echo(
+                click.style(
+                    f"Input File Size is {os.stat(infile.name).st_size / (1024*1024):.2f} MB, it may take a while to process. CTRL+C to stop.",
+                    fg="yellow",
+                    bold=True,
+                ),
+                err=True,
+            )
+
+        with open(infile.name, "r", encoding="utf-8", errors="ignore") as f:
+            total_lines = sum(1 for _ in f)
+
+        kwargs["total"] = total_lines if not disable else 1
+        super().__init__(**kwargs)
+
+    def update_with_result(
+        self, result, field="id", error_resource_type=None, error_parameter="ids"
+    ):
+        """
+        Update the progress bar appropriately, with a full API response. For convenience,
+        and drop in compatibility with FileSizeProgressBar otherwise use tqdm's update().
+        """
+
+        try:
+            if "data" in result:
+                for item in result["data"]:
+                    self.update()
+            if error_resource_type and "errors" in result:
+                for error in result["errors"]:
+                    # Account for deleted data
+                    # Errors have very inconsistent format, missing fields for different types of errors...
+                    if (
+                        "resource_type" in error
+                        and error["resource_type"] == error_resource_type
+                    ):
+                        if (
+                            "parameter" in error
+                            and error["parameter"] == error_parameter
+                        ):
+                            self.update()
+                            # todo: hide or show this?
+                            # self.set_description(
+                            #    "Errors encountered, results may be incomplete"
+                            # )
+                        # print(error["value"], error["resource_type"], error["parameter"])
+        except Exception as e:
+            log.error(f"Failed to update progress bar: {e}")
+
+
 class FileSizeProgressBar(tqdm):
     """
     An input file size based progress bar. Counts an input file in bytes.
@@ -224,6 +319,10 @@ class FileSizeProgressBar(tqdm):
     def update_with_result(
         self, result, field="id", error_resource_type=None, error_parameter="ids"
     ):
+        """
+        Update the progress bar appropriately, with a full API response. For convenience,
+        otherwise use twdm's own update() method.
+        """
         try:
             if "data" in result:
                 for item in result["data"]:
@@ -318,7 +417,7 @@ class TimestampProgressBar(tqdm):
 
     def update_with_result(self, result):
         """
-        Update progress bar based on snowflake ids.
+        Update progress bar based on snowflake ids from an API response.
         """
         try:
             newest_id = result["meta"]["newest_id"]
